//-------------------------------------------------------------------------------------------------
// Image.cpp (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "Image.hpp"
#include "engineAssert.hpp"
using namespace std;


//-------------------------------------------------------------------------------------------------
// Static member initialization
//-------------------------------------------------------------------------------------------------
STATIC byte_t	Image::_primerTGA_correct[ 12 ] = {0,0,2,0,0,0,0,0,0,0,0,0}; // FIXME: put this somewhere else


//-------------------------------------------------------------------------------------------------
// Class constants
//-------------------------------------------------------------------------------------------------
//const dword_t Image::TEXNUM_UNREGISTERED = 0xffffffff;


//-------------------------------------------------------------------------------------------------
// Constructors
//-------------------------------------------------------------------------------------------------
Image::Image( const std::string& filePath )
:
_filePath( filePath ),
_width( 0 ),
_height( 0 ),
_bpp( 0 ),
_pixelData( NULL ),
_internalFormat( 0 ),
_bufferFormat( 0 )
//_glTexNum( TEXNUM_UNREGISTERED )
{
	_LoadFromFile( filePath );
}


//-------------------------------------------------------------------------------------------------
// Destructor
//-------------------------------------------------------------------------------------------------
Image::~Image()
{
}


//-------------------------------------------------------------------------------------------------
// _LoadFromFile
//-------------------------------------------------------------------------------------------------
void Image::_LoadFromFile( const std::string& filePath )
{
	/// Open the file
	FILE* file = fopen( filePath.c_str(), "rb" );
	VALIDATE_MSG1( file, "Image::LoadFromFile : failed to open \"%s\"\n", filePath.c_str() );

	/// Load TGA file and close
	_LoadTGAFromFile( file );
	fclose( file );
}


//-------------------------------------------------------------------------------------------------
// _LoadTGAFromFile
//-------------------------------------------------------------------------------------------------
void Image::_LoadTGAFromFile( FILE* file )
{
	/// Read in the 12-byte primer and validate it
	byte_t primerTGA[ 12 ];
	int bytesRead = fread( primerTGA, 1, sizeof( primerTGA ), file );
	VALIDATE( bytesRead == sizeof( primerTGA ) );
	VALIDATE( !memcmp( primerTGA, _primerTGA_correct, sizeof( primerTGA ) ) );

	/// Read in the 6-byte header core
	byte_t dummyByte;
	fread( &_width,		sizeof( _width ),		1, file ); // short
	fread( &_height,	sizeof( _height ),		1, file ); // short
	fread( &_bpp,		sizeof( _bpp ),			1, file ); // byte
	fread( &dummyByte,	sizeof( dummyByte ),	1, file ); // byte

	/// Set the proper opengl image format
	if( _bpp == 32 )
	{
		_internalFormat = GL_RGBA;
		_bufferFormat = GL_BGRA_EXT;
	}
	else if( _bpp == 24 )
	{
		_internalFormat = GL_RGB;
		_bufferFormat = GL_BGR_EXT;
	}
	else
	{
		PANIC_MSG2( "Image::_LoadTGAFromFile : %s had unsupported bit-depth of %d bpp\n", _filePath.c_str(), _bpp );
	}

	/// Calculate pixel data size and <bytesPerPixel>
	int bytesPerPixel = _bpp / 8;
	int pixelBytes = _width * _height * bytesPerPixel;

	/// Allocate memory for the data segment
	_pixelData = new byte_t[ pixelBytes ];

	/// Read in the data segment (pixels in BGR format)
	bytesRead = fread( _pixelData, sizeof( byte_t ), pixelBytes, file ); // read in BGR data
	VALIDATE( bytesRead == pixelBytes );
}


