//-------------------------------------------------------------------------------------------------
// TheInputSystem.hpp (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#ifndef _include_TheInputSystem_
#define _include_TheInputSystem_


//-------------------------------------------------------------------------------------------------
// Included header files
//-------------------------------------------------------------------------------------------------
#include "engineCommon.hpp"


///////////////////////////////////////////////////////////////////////////////////////////////////
//
//		TheInputSystem
//
///////////////////////////////////////////////////////////////////////////////////////////////////
class TheInputSystem
{
private:
	/// Static members
	static bool _isInitialized;

	/// Private members
	bool	_isKeyDown[ 0xff ];
	int		_mouseDeltaX;
	int		_mouseDeltaY;
	int		_mousePosX;
	int		_mousePosY;
	bool	_useWindowsMouseCursor;
	bool	_doesAppHaveFocus;

public:
	enum inputEventType_e
	{
		INPUT_EVENT_NONE,
		INPUT_EVENT_KEYDOWN,
		INPUT_EVENT_KEYREPEAT,
		INPUT_EVENT_KEYUP,
		INPUT_EVENT_MOUSEWHEEL_UP,
		INPUT_EVENT_MOUSEWHEEL_DOWN,
	};

	/// Construction, initialization, and singleton methods
	TheInputSystem(); // default constructor
	~TheInputSystem(); // destructor

	void Startup();
	void Shutdown();

	/// Public methods
	void Update();
	void UseWindowsMouseCursor( const bool enable );
	const bool IsKeyDown( byte_t vKeyCode ) const;
	const bool ProcessInputEvent( const UINT messageID, const WPARAM wParam, const LPARAM lParam );
	void GetMousePosScreenAbsolute( int& screenMousePosX, int& screenMousePosY ) const;
	void SetMousePosScreenAbsolute( const int screenMousePosX, const int screenMousePosY );
	void GetMousePosClientAbsolute( int& clientMousePosX, int& clientMousePosY ) const;
	void SetMousePosClientAbsolute( const int clientMousePosX, const int clientMousePosY );
	void GetMouseDeltas( int& mouseDeltaX, int& mouseDeltaY );
	int GetMouseDeltaX();
	int GetMouseDeltaY();
	void OnAppGainFocus();
	void OnAppLoseFocus();
};


//-------------------------------------------------------------------------------------------------
// Global singleton instance
//-------------------------------------------------------------------------------------------------
extern TheInputSystem theInputSystem;


#endif // _include_TheInputSystem_
