//-------------------------------------------------------------------------------------------------
// TheNetwork.hpp (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#ifndef _include_TheNetwork_
#define _include_TheNetwork_


//-------------------------------------------------------------------------------------------------
// Included header files
//-------------------------------------------------------------------------------------------------
#include "engineCommon.hpp"


///////////////////////////////////////////////////////////////////////////////////////////////////
//
//		TheNetwork
//
///////////////////////////////////////////////////////////////////////////////////////////////////
class TheNetwork
{
private:
	enum
	{
		DEFAULT_PORT_NUMBER			= 50000, // 51726,
		DEFAULT_MAX_CONNECTIONS		= 32,
		MAX_MS_THREAD_LOCK			= 10, // Don't spend more than X ms in a critical section
		MAX_HOSTNAME_LEN			= 256,
	};

	struct HostConnection
	{
		unsigned int		localConnectionID;
		struct sockaddr		hostInfo;
		SOCKET				sock;
	};

	/// Static members
	static TheNetwork*				_TheNetwork;
	static bool						_isInitialized;
	static CRITICAL_SECTION			_criticalSection_connectionList;
	static unsigned int				_nextConnectionID;

	/// Private members
	bool							_isNetworkServiceStarted;
	int								_portNumber;
	int								_maxNumberOfConnections;
	WSADATA							_winSockVersionData;
	std::string						_localHostName;
	struct hostent*					_localHostInfo;
	struct sockaddr_in				_localSockAddr;
	SOCKET							_listenSocket;
	std::vector<HostConnection>		_connectionList;

	/// Private static methods
	static void _ListenThreadEntryPoint( void* data );
	static void _RecvThreadEntryPoint( void* data );

	/// Private methods
	void _InitializeWinSock();
	void _ShutDownWinSock();
	void _CreateListenSocket();
	void _SetLocalHostName( const std::string& localHostName ) { _localHostName = localHostName; }
	std::string _GetLocalHostName() const { return _localHostName; }
	void _StartListenThread();
	void _StartRecvThread( unsigned int connectionID );
	void _ListenThreadDoWork( void* data );
	void _RecvThreadDoWork( void* data );
	void _HandleSocketSelectError( int errorCode );
	void _HandleSocketAcceptError( int errorCode );
	void _HandleNewSocket( SOCKET newSocket, struct sockaddr newHostInfo );
	unsigned int _GetNewUniqueLocalConnectionID();

public:
	TheNetwork(); // default constructor
	~TheNetwork(); // destructor

	/// Public methods
	void Startup();
	void Shutdown();

	void StartNetworkServices();
	void StopNetworkServices();
};


//-------------------------------------------------------------------------------------------------
// Global singleton instance
//-------------------------------------------------------------------------------------------------
extern TheNetwork theNetwork;


#endif // _include_TheNetwork_
