//-------------------------------------------------------------------------------------------------
// engineCommon.hpp (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
// 
// Included by all engine headers.
//-------------------------------------------------------------------------------------------------
#ifndef _include_engineCommon_
#define _include_engineCommon_


//-------------------------------------------------------------------------------------------------
// General-purpose typedefs
//-------------------------------------------------------------------------------------------------
typedef unsigned char byte_t;
typedef unsigned short word_t;
typedef unsigned long dword_t;
typedef unsigned long bitflags_t;


//-------------------------------------------------------------------------------------------------
// General-purpose constants
//-------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------
// General-purpose macros
//-------------------------------------------------------------------------------------------------
#define UNUSED_ARG	(void)
#define VIRTUAL
#define STATIC
#undef NULL
#define NULL		(0)
#define BIT(x)		(1<<(x))


//-------------------------------------------------------------------------------------------------
// Things that Windows should define for us but doesn't for some unknown reason
//-------------------------------------------------------------------------------------------------
#ifndef VK_OEM_1          
#define VK_OEM_1          0xBA   // ';:' for US
#endif
#ifndef VK_COLON          
#define VK_COLON          0xBA   // ';:' for US
#endif
#ifndef VK_OEM_PLUS       
#define VK_OEM_PLUS       0xBB   // '+' any country
#endif
#ifndef VK_OEM_COMMA      
#define VK_OEM_COMMA      0xBC   // ',' any country
#endif
#ifndef VK_OEM_MINUS      
#define VK_OEM_MINUS      0xBD   // '-' any country
#endif
#ifndef VK_OEM_PERIOD     
#define VK_OEM_PERIOD     0xBE   // '.' any country
#endif
#ifndef VK_OEM_2          
#define VK_OEM_2          0xBF   // '/?' for US
#endif
#ifndef VK_SLASH          
#define VK_SLASH          0xBF   // '/?' for US
#endif
#ifndef VK_OEM_3          
#define VK_OEM_3          0xC0   // '`~' for US
#endif
#ifndef VK_TILDE          
#define VK_TILDE          0xC0   // '`~' for US
#endif
#ifndef VK_OEM_4          
#define VK_OEM_4          0xDB  //  '[{' for US
#endif
#ifndef VK_LEFTBRACKET          
#define VK_LEFTBRACKET    0xDB  //  '[{' for US
#endif
#ifndef VK_OEM_5          
#define VK_OEM_5          0xDC  //  '\|' for US
#endif
#ifndef VK_BACKSLASH          
#define VK_BACKSLASH      0xDC  //  '\|' for US
#endif
#ifndef VK_OEM_6          
#define VK_OEM_6          0xDD  //  ']}' for US
#endif
#ifndef VK_RIGHTBRACKET   
#define VK_RIGHTBRACKET   0xDD  //  ']}' for US
#endif
#ifndef VK_OEM_7          
#define VK_OEM_7          0xDE  //  ''"' for US
#endif
#ifndef VK_QUOTE          
#define VK_QUOTE          0xDE  //  ''"' for US
#endif


//-------------------------------------------------------------------------------------------------
// General-purpose functions (cpp)
//-------------------------------------------------------------------------------------------------
int Sprintf( char* buffer, size_t maxLength, const char* format, ... );
void DebugPrintf( const char* format, ... );


///////////////////////////////////////////////////////////////////////////////////////////////////
//
//		General-purpose functions (inline)
//
///////////////////////////////////////////////////////////////////////////////////////////////////
//-------------------------------------------------------------------------------------------------
// Min functions
//-------------------------------------------------------------------------------------------------
inline int Min( int a, int b )
{
	return a < b ? a : b;
}
inline float Min( float a, float b )
{
	return a < b ? a : b;
}
inline double Min( double a, double b )
{
	return a < b ? a : b;
}
//-------------------------------------------------------------------------------------------------
// Max functions
//-------------------------------------------------------------------------------------------------
inline int Max( int a, int b )
{
	return a > b ? a : b;
}
inline float Max( float a, float b )
{
	return a > b ? a : b;
}
inline double Max( double a, double b )
{
	return a > b ? a : b;
}


//-------------------------------------------------------------------------------------------------
// General header files
//-------------------------------------------------------------------------------------------------
#include "engineSysHeaders.hpp"


#endif // _include_engineCommon_
