//-------------------------------------------------------------------------------------------------
// engineMath.hpp (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
// 
// Common suite of math classes.
//-------------------------------------------------------------------------------------------------
#ifndef _include_engineMath_
#define _include_engineMath_


//-------------------------------------------------------------------------------------------------
// General-purpose math constants
//-------------------------------------------------------------------------------------------------
const double PI = 3.1415926535897932384626433832795;
const double HALF_PI = 1.5707963267948966192313216916398;
const double TWO_PI = 6.283185307179586476925286766559;
const double E = 2.7182818284590452353602874713527;


//-------------------------------------------------------------------------------------------------
// General-purpose math macros
//-------------------------------------------------------------------------------------------------
//#define DEG2RAD(x)	(x * 0.017453292519943295769236907684886)
//#define RAD2DEG(x)	(x * 57.295779513082320876798154814105)
inline float DEG2RAD( float x )
{
	return (x * 0.017453292519943295769236907684886f);
}

inline double DEG2RAD( double x )
{
	return (x * 0.017453292519943295769236907684886);
}

inline float RAD2DEG( float x )
{
	return (x * 57.295779513082320876798154814105f);
}

inline double RAD2DEG( double x )
{
	return (x * 57.295779513082320876798154814105);
}


//-------------------------------------------------------------------------------------------------
// General-purpose inline math utility functions
//-------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------
// ClampFloat
// 
// Return <value> clamped to the range [min,max].
// NOTE: this function assumes that max >= min
//-------------------------------------------------------------------------------------------------
inline float ClampFloat( float value, float min, float max )
{
	if( value > max )
	{
		return max;
	}
	else if( value < min )
	{
		return min;
	}
	else
	{
		return value;
	}
}


//-------------------------------------------------------------------------------------------------
// ClampDouble
// 
// Return <value> clamped to the range [min,max].
// NOTE: this function assumes that max >= min
//-------------------------------------------------------------------------------------------------
inline double ClampDouble( double value, double min, double max )
{
	if( value > max )
	{
		return max;
	}
	else if( value < min )
	{
		return min;
	}
	else
	{
		return value;
	}
}


//-------------------------------------------------------------------------------------------------
// MaxFloat
//-------------------------------------------------------------------------------------------------
inline float MaxFloat( float value1, float value2 )
{
	if( value1 > value2 )
		return value1;
	else
		return value2;
}


//-------------------------------------------------------------------------------------------------
// MaxFloat
//-------------------------------------------------------------------------------------------------
inline float MaxFloat( float value1, float value2, float value3 )
{
	if( value1 > value2 )
	{
		if( value1 > value3 )
			return value1;
		else
			return value3;
	}
	else
	{
		if( value2 > value3 )
			return value2;
		else
			return value3;
	}
}


//-------------------------------------------------------------------------------------------------
// MaxFloat
//-------------------------------------------------------------------------------------------------
inline float MaxFloat( float value1, float value2, float value3, float value4 )
{
	if( value1 > value2 )
	{
		if( value3 > value4 )
		{
			if( value1 > value3 )
				return value1;
			else
				return value3;
		}
		else
		{
			if( value1 > value4 )
				return value1;
			else
				return value4;
		}
	}
	else
	{
		if( value3 > value4 )
		{
			if( value2 > value3 )
				return value2;
			else
				return value3;
		}
		else
		{
			if( value2 > value4 )
				return value2;
			else
				return value4;
		}
	}
}


//-------------------------------------------------------------------------------------------------
// MinFloat
//-------------------------------------------------------------------------------------------------
inline float MinFloat( float value1, float value2 )
{
	if( value1 < value2 )
		return value1;
	else
		return value2;
}


//-------------------------------------------------------------------------------------------------
// MinFloat
//-------------------------------------------------------------------------------------------------
inline float MinFloat( float value1, float value2, float value3 )
{
	if( value1 < value2 )
	{
		if( value1 < value3 )
			return value1;
		else
			return value3;
	}
	else
	{
		if( value2 < value3 )
			return value2;
		else
			return value3;
	}
}


//-------------------------------------------------------------------------------------------------
// MinFloat
//-------------------------------------------------------------------------------------------------
inline float MinFloat( float value1, float value2, float value3, float value4 )
{
	if( value1 < value2 )
	{
		if( value3 < value4 )
		{
			if( value1 < value3 )
				return value1;
			else
				return value3;
		}
		else
		{
			if( value1 < value4 )
				return value1;
			else
				return value4;
		}
	}
	else
	{
		if( value3 < value4 )
		{
			if( value2 < value3 )
				return value2;
			else
				return value3;
		}
		else
		{
			if( value2 < value4 )
				return value2;
			else
				return value4;
		}
	}
}


//-------------------------------------------------------------------------------------------------
// RandomFloat
//-------------------------------------------------------------------------------------------------
inline float RandomFloat()
{
	return (float) rand() / (float) RAND_MAX;
}


//-------------------------------------------------------------------------------------------------
// RandomFloat
//-------------------------------------------------------------------------------------------------
inline float RandomFloat( float max )
{
	return max * ((float) rand() / (float) RAND_MAX);
}


//-------------------------------------------------------------------------------------------------
// RandomFloat
//-------------------------------------------------------------------------------------------------
inline float RandomFloat( float min, float max )
{
	float range = max - min;
	return min + range * ((float) rand() / (float) RAND_MAX);
}


//-------------------------------------------------------------------------------------------------
// Included header files
//-------------------------------------------------------------------------------------------------
#include "engineCommon.hpp"


#endif // _include_engineMath_
