//-------------------------------------------------------------------------------------------------
// Baby.hpp
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "Animation.hpp"


class Baby
{
public:
	static const dword_t		BABY_THINK_DELAY ;
	static const float			BABY_CRAWL_SPEED;
	static const float			BABY_RADIUS;
	static const char*			BABY_ANIM_NAME_IDLE;
	static const char*			BABY_ANIM_NAME_CRAWL;
	static const char*			BABY_ANIM_NAME_WOUNDEDIDLE;
	static const char*			BABY_ANIM_NAME_WOUNDEDCRAWL;
	static const char*			BABY_ANIM_NAME_DEAD ;
	static const char*			BABY_ANIM_NAME_HEART ;
	static const unsigned int	BABY_STARTING_HEARTS ;
	static std::vector<Baby*> babyList;

	typedef enum
	{
		BABY_STATE_ALIVE,
		BABY_STATE_DEAD,
	} BabyState ;

	/// Construction / deconstruction
	Baby( const Vector2& startPosition, float startFacingDegrees );
	~Baby();

	void			concussionPoint( Vector2& impactPoint, float radius );

	/// Updating, drawing, etc.
	static void		UpdateAll();
	void			Update();

	static void		DrawAll();
	void			Draw();

	static bool		BabiesAllDead();

protected:
	void			_think( void );
	void			_move( void );
	void			_pickNewGoal( void );
	bool			_isGoalAccessible( Vector2& goalPosition );
	bool			_isPositionValid( unsigned int x, unsigned int y );
	void			_checkForCollisions( void );

private:
	BabyState		_babyState ;
	unsigned int	_babyNumber ;
	dword_t			_lastThinkTime;
	Vector2			_goalPosition ;
	float			_speed ;
	Vector2			_directionVector ;
	Vector2			position;
	float			_headingDegrees;
	unsigned int	_hearts;
	Animation*		anim_idle;
	Animation*		anim_crawl;
	Animation*		anim_woundedCrawl;
	Animation*		anim_heart ;
	Animation*		anim_dead ;
};




/*

{
	Vector2 center( 30.0f, 50.0f );
	anim_idle->Draw( center );
}



{
	anim_idle = new Animation( "BabyAnim_Idle.tga" );
	anim_idle = new Animation( "BabyAnim_Crawl.tga" );
	anim_idle = new Animation( "BabyAnim_woundedCrawl.tga" );
}



Animation::Animation( const char* fileName )
{
	m_texture = new Texture( fileName );
	width
	height

	numberOfFrames
	currentFrame
}


Animation::Draw( const Vector2& screenCenterCoordinates )
{
	/// figure out what frame we're on

	/// figure out what texture coordinates are needed for that frame

	/// draw using the renderer and texture coordinates
	theRenderer.DrawTexturedRectCentered( m_texture, screenCenterCoordinates );
}


*/