// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.

#include "pch.hpp"
#include "Mine.hpp"
using namespace std;


//-------------------------------------------------------------------------------------------------
// Class constants
//-------------------------------------------------------------------------------------------------
STATIC const float Mine::MINE_RADIUS = 10.0f;
STATIC const char* Mine::MINE_ANIM_NAME_IDLE = "Mine/Mine_idle.tga";
STATIC std::vector<Mine*> Mine::mineList;


//-------------------------------------------------------------------------------------------------
// Constructor
//-------------------------------------------------------------------------------------------------
Mine::Mine( const Vector2& startPosition )
:
//anim_idle( animIdlePath ),
position( startPosition )
{
	/// Register myself with the master list
	mineList.push_back( this );

	anim_idle = new Animation( "Misc\\landmine01.tga", 5 );
}


//-------------------------------------------------------------------------------------------------
// Destructor
//-------------------------------------------------------------------------------------------------
Mine::~Mine()
{
	/// Remove myself from the master list
	std::vector<Mine*>::iterator iter;
	for( iter = mineList.begin(); iter != mineList.end(); iter ++ )
	{
		if( *iter == this )
		{
			mineList.erase( iter );
			break;
		}
	}
}


//-------------------------------------------------------------------------------------------------
// UpdateAll
//-------------------------------------------------------------------------------------------------
STATIC void Mine::UpdateAll()
{
	std::vector<Mine*>::iterator iter;
	for( iter = mineList.begin(); iter != mineList.end(); iter ++ )
	{
		Mine* mine = *iter;
		mine->Update();
	}
}


//-------------------------------------------------------------------------------------------------
// Update
//-------------------------------------------------------------------------------------------------
void Mine::Update()
{
}


//-------------------------------------------------------------------------------------------------
// DrawAll
//-------------------------------------------------------------------------------------------------
STATIC void Mine::DrawAll()
{
	std::vector<Mine*>::iterator iter;
	for( iter = mineList.begin(); iter != mineList.end(); iter ++ )
	{
		Mine* mine = *iter;
		mine->Draw();
	}
}


//-------------------------------------------------------------------------------------------------
// Draw
//-------------------------------------------------------------------------------------------------
void Mine::Draw()
{
	//theRenderer.DrawCircle( position, MINE_RADIUS, Rgba::RED );
	//anim_idle->Draw(position, Vector2::ONE, (float)theApplication.GetTimeFrameBegan() * 300);
	anim_idle->Draw(position, Vector2::ONE, (float)theApplication.GetTimeFrameBegan() * 300);
}




STATIC Mine* Mine::FindFirstMineThatCollidesWithPoint( Vector2& position, float radius )
{
	std::vector<Mine*>::iterator iter ;
	for( iter = mineList.begin(); iter != mineList.end(); ++iter )
	{
		Mine* mine = (*iter);
		if ( mine->Collide( position, radius ) )
		{
			return mine ;
		}
	}

	return NULL ;
}



bool Mine::Collide( Vector2& collisionPoint, float radius )
{
	Vector2 differenceVector = collisionPoint - position ;
	if ( differenceVector.CalcLength() < radius + MINE_RADIUS )
		return true ;
	return false ;
}


