//---------------------------------------------------------
// TheMap -- Definition of TheMap class
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//---------------------------------------------------------

#ifndef _THE_MAP_
#define _THE_MAP_

//forward declarations
class TheMap ;

// Includes
#include "..\Engine\Image.hpp"
#include "..\Engine\Texture.hpp"
#include "Animation.hpp"
#include <string>


#define LEVEL_DIRECTORY "levels\\"

typedef enum
{
	TERRAIN_CLEAR		= 0,
	TERRAIN_BLOCKED		= 1,
	TERRAIN_MUD			= 2,
	TERRAIN_ICE			= 3,
	TERRAIN_START		= 4,
} TerrainType ;



//---------------------------------------------------------
// A single map cell
//---------------------------------------------------------
struct _mapCell
{
	TerrainType			_terrainType ;
};

typedef struct _mapCell MapCell ;



//----------------------------------------------------------
// Holds an instance of the level map.
//----------------------------------------------------------
class TheMap
{
public:

	static const unsigned int MAP_WIDTH ;
	static const unsigned int MAP_HEIGHT ;
	static const unsigned int BABY_START ;
	static const unsigned int TOWER_SPAWN ;
	static const unsigned int MINE_SPAWN ;
	static const unsigned int GRINDER_SPAWN ;
	static const unsigned int OBSTACLE ;
	static const unsigned int MUD ;
	static const unsigned int ICE ;

	TheMap( void );
	~TheMap( void );

	// Queries
	bool			isBlocked( unsigned int x, unsigned int y );
	bool			isBlocked( Vector2& position );
	bool			tracePath( Vector2& startPosition, Vector2& endPosition, float radius );

	// Accessors
	TerrainType		getTerrainAt( Vector2& position );
	TerrainType		getTerrainAt( unsigned int x, unsigned int y );

	void			loadLevel( const std::string& name );
	bool			update( void );
	void			draw( void );


protected:
	void			_spawnTower( unsigned int x, unsigned int y );
	void			_spawnMine( unsigned int x, unsigned int y );
	void			_spawnBaby( unsigned int x, unsigned int y );

	void			_parseMapData( void );
	bool			_isPositionValid( unsigned int x, unsigned int y, float radius );
	void			_init( void );


private:
	MapCell*		_mapData ;
	Animation*		_mapBackground ;
	Image*			_mapImageData ;
	bool			_isInitialized ;
	std::string		_name ;
};



//externs
extern TheMap theMap ;

#endif /* _THE_MAP_ */





