//------------------------------------------------------------------------------------------------
// Tower.cpp -- Implementation of Tower class
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//------------------------------------------------------------------------------------------------

#include "pch.hpp"
#include "Tower.hpp"

using namespace std;


//-------------------------------------------------------------------------------------------------
// Class constants
//-------------------------------------------------------------------------------------------------
STATIC const float Tower::TOWER_RADIUS = 10.0f;
STATIC const char* Tower::TOWER_ANIM_NAME_IDLEFIRE = "Turret/turret_base.tga";
STATIC std::vector<Tower*> Tower::towerList;


//-------------------------------------------------------------------------------------------------
// Constructor
//-------------------------------------------------------------------------------------------------
Tower::Tower( const Vector2& startPosition )
	:	anim_idleFire( NULL ),
		position( startPosition )
{
	/// Register myself with the master list
	towerList.push_back( this );

	anim_idleFire = new Animation( TOWER_ANIM_NAME_IDLEFIRE );
}


//-------------------------------------------------------------------------------------------------
// Destructor
//-------------------------------------------------------------------------------------------------
Tower::~Tower()
{
	/// Remove myself from the master list
	std::vector<Tower*>::iterator iter;
	for( iter = towerList.begin(); iter != towerList.end(); iter ++ )
	{
		if( *iter == this )
		{
			towerList.erase( iter );
			break;
		}
	}

	if ( anim_idleFire )
	{
//		delete anim_idleFire ;
		anim_idleFire = NULL ;
	}
}


//-------------------------------------------------------------------------------------------------
// UpdateAll
//-------------------------------------------------------------------------------------------------
STATIC void Tower::UpdateAll()
{
	std::vector<Tower*>::iterator iter;
	for( iter = towerList.begin(); iter != towerList.end(); iter ++ )
	{
		Tower* tower = *iter;
		tower->Update();
	}
}


//-------------------------------------------------------------------------------------------------
// Update
//-------------------------------------------------------------------------------------------------
void Tower::Update()
{
}


//-------------------------------------------------------------------------------------------------
// DrawAll
//-------------------------------------------------------------------------------------------------
STATIC void Tower::DrawAll()
{
	std::vector<Tower*>::iterator iter;
	for( iter = towerList.begin(); iter != towerList.end(); iter ++ )
	{
		Tower* tower = *iter;
		tower->Draw();
	}
}


//-------------------------------------------------------------------------------------------------
// Draw
//-------------------------------------------------------------------------------------------------
void Tower::Draw()
{
	theRenderer.DrawCircle( position, TOWER_RADIUS, Rgba::YELLOW );
	anim_idleFire->Draw(position, Vector2( 0.5f, 0.5f ), 0.0f, 0 );//(float)theApplication.GetTimeFrameBegan() * 300);
}





